@@ -9,15 +9,15 @@ use Scalar::Util qw(blessed refaddr weaken);
 sub new {
     my ($class) = @_;
     my $self = &share({});
-    bless $self, $class;
+    bless($self, $class);
     return $self;
 }
 
 sub insert : locked method {
     my ($self, $obj) = @_;
     my $ref = _encode_ref($obj);
-    $self->{$ref} = $obj;
-    weaken $self->{$ref};
+    $self->{$ref} = &share($obj);
+    weaken($self->{$ref});
     _log "insert $obj as $ref\n";
     return $ref;
 }
@@ -32,20 +32,19 @@ sub get : locked method {
 
 sub find : locked method {
     my ($self, $class) = @_;
-    my $obj;
-    for my $ref (keys %$self) {
-        $obj = $self->{$ref};
-        last if $class eq blessed $obj;
-        $obj = undef;
+    my $ref;
+    for (keys %$self) {
+        $ref = $_;
+        last if $class eq blessed($self->{$ref});
     }
     no warnings 'uninitialized';
-    _log "find $obj as $class\n";
-    return $obj;
+    _log "find $ref as $class\n";
+    return $ref;
 }
 
 sub _encode_ref {
     my ($obj) = @_;
-    return refaddr $obj;
+    return refaddr($obj);
 }
 
 1;
